/* Copyright (C) 2005, 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package backend;

/**
 * LittleEInstr: default implementation for Instruction on little endian architectures
 * 
 * @author S. Frenz
 * @version 090207 added copyright notice
 *  version 060607 initial version
 */

public class LittleEInstr extends Instruction { //Little Endian
  public LittleEInstr(int maxInstrSize) {
    super();
    code=new byte[maxInstrSize]; //maximum length of a single instruction
  }
  
  public void replaceShort(int off, int val) {
    code[off++]=(byte)val;
    code[off  ]=(byte)(val>>8);
  }
  
  public void replaceInt(int off, int val) {
    code[off++]=(byte)val;
    code[off++]=(byte)(val>>8);
    code[off++]=(byte)(val>>16);
    code[off  ]=(byte)(val>>24);
  }
  
  public void replaceLong(int off, long val) {
    code[off++]=(byte)val;
    code[off++]=(byte)(val>>8);
    code[off++]=(byte)(val>>16);
    code[off++]=(byte)(val>>24);
    code[off++]=(byte)(val>>32);
    code[off++]=(byte)(val>>40);
    code[off++]=(byte)(val>>48);
    code[off  ]=(byte)(val>>56);
  }
}
