/* Copyright (C) 2005, 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package compbase;

/**
 * Modifier: platform and language independent modifiers for units and methods 
 * 
 * @author S. Frenz
 * @version 090718 added MF_ISWRITTEN and MF_MAYBEWRITTEN
 *  version 090218 added synchronized and volatile modifier flag
 *  version 090207 added copyright notice
 *  version 080706 added MA_INTFMD
 *  version 080121 added transient modifier flag
 *  version 080118 added annotation modifier flag
 *  version 071001 added M_NATIVE and M_MSWIN
 *  version 070527 added more specials
 *  version 070521 added more specials
 *  version 070509 added more specials
 *  version 070114 added more specials
 *  version 061228 added more specials
 *  version 061211 added more specials
 *  version 061030 added specials
 *  version 060607 initial version
 */

public class Modifier {
	public final static int M_PUB   = 0x0001; //"public"
	public final static int M_PROT  = 0x0002; //"protected"
	public final static int M_PRIV  = 0x0004; //"private"
	public final static int M_FIN   = 0x0008; //"final"
	public final static int M_STAT  = 0x0010; //"static"
	public final static int M_ABSTR = 0x0020; //"abstract"
  public final static int M_NAT   = 0x0040; //"native"
  public final static int M_MSWIN = 0x0080; //windows dll-call
  public final static int M_TRANS = 0x0100; //"transient"
  public final static int M_ANNO  = 0x0200; //"@" for annotation modifier
  public final static int M_SYNC  = 0x0400; //"synchronized"
  public final static int M_VOLAT = 0x0800; //"volatile"
  //following: unit specials
  public final static int M_INDIR  = 0x01000; //special mark for units: indirect call (i.e. interfaces)
  public final static int M_STRUCT = 0x02000; //special mark for units: struct
  public final static int M_ARRLEN = 0x04000; //special mark for vrbls and units: length of inline array
  //following: method specials
  public final static int M_EXINIT = 0x08000; //special mark for mthds: contains explicit init
  public final static int M_OVERLD = 0x10000; //special mark for mthds: is overloaded
  public final static int M_HSCALL = 0x20000; //special mark for mthds: method contains call
  public final static int M_NDDESC = 0x40000; //special mark for mthds: need descriptor entry beside normal mode
  public final static int M_NDCODE = 0x80000; //special mark for mthds: generate code
  //following: access specials
  public final static int MA_ACCSSD = 0x100000; //is accessed
  public final static int MA_PROT   = 0x200000; //access through "protected" way
  public final static int MA_PUB    = 0x400000; //access through "public" way
  public final static int MA_INTFMD = 0x800000; //access through interface map
  //following: flow analysis
  public final static int MF_ISWRITTEN    = 0x1000000; //variable is written
  public final static int MF_MAYBEWRITTEN = 0x2000000; //variable may be written
}
