/* Copyright (C) 2005, 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package compbase;

/**
 * StringPool: pool for strings re-usage, may reduce memory requirements
 * 
 * @author S. Frenz
 * @version 090207 added copyright notice
 *  version 060608 inserted different pooling strategies
 *  version 060607 initial version
 */

public class StringPool {
	//private StringList s, last; //for pools based on StringList
  private String str; //for pools with tree
  private StringPool left, right; //for pools with binary tree
	
  //pool as binary tree (17687839 ticks with 9333 kb)
  public String getString(char buf[], int offset, int len) {
    int i,  diff=0, max, strLen;
    
    if (str==null) {
      str=new String(buf, offset, len);
      return str;
    }
    strLen=str.length();
    if (len<=strLen) max=len;
    else max=strLen;
    for (i=0; i<max; i++) {
      if ((diff=(int)(buf[i+offset]-str.charAt(i)))!=0) break; //different character
    }
    if (i==len && i==strLen) return str;
    if (diff<0) {
      if (right==null) right=new StringPool();
      return right.getString(buf, offset, len);
    }
    if (left==null) left=new StringPool();
    return left.getString(buf, offset, len);
  }
  
  //no pooling (15877454 ticks with 11602 kb)
  /*public String getString(char buf[], int offset, int len) {
    return new String(buf, offset, len);
  }*/
  
  //pool with linear StringPool and re-insertion (21294959 ticks with 9333 kb)
  /*public String getString(char buf[], int offset, int len) {
    int i;
    String to;
    StringList t=s, told=null;
    
    while (t!=null) {
      to=t.str;
      if (to.length()==len) {
        for (i=0; i<len; i++) if (buf[i+offset]!=to.charAt(i)) break; //different character
        if (i==len) {
          if (told!=null) {
            if (t==last) last=told;
            told.next=t.next;
            t.next=s;
            s=t;
          }
          return t.str;
        }
      }
      told=t;
      t=t.next;
    }
    to=new String(buf, offset, len);
    last=new StringList(last, to);
    if (s==null) s=last;
    return to;
  }*/
  
  //pool with linear StringList (36414160 ticks with 9333 kb)
	/*public String getString(char buf[], int offset, int len) {
		int i;
		String to;
		StringList t=s;
		
		while (t!=null) {
			to=t.str;
			if (to.length()==len) {
				for (i=0; i<len; i++) if (buf[i+offset]!=to.charAt(i)) break; //different character
				if (i==len) return t.str;
			}
			t=t.next;
		}
		to=new String(buf, offset, len);
		last=new StringList(last, to);
		if (s==null) s=last;
		return to;
	}*/
}
