/* Copyright (C) 2005, 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package frontend.sjava;

import compbase.*;

/**
 * StRetMissing: pseudo-statement to handle "return missing"
 * 
 * @author S. Frenz
 * @version 090619 adopted changed Architecture
 *  version 090508 adopted changes in Stmt
 *  version 090506 initial version
 */

public class StRetMissing extends Stmt {
  private UnitList runtimeClass;
  
	protected StRetMissing(int fid, int il, int ic) {
		super(fid, il, ic);
	}
	
	protected int resolve(int flowCode, Unit unitContext, Mthd mthdContext, Context ctx) {
    if ((flowCode&FA_NEXT_IS_UNREACHABLE)==0) flowWarn(ctx, "reached end of method without return");
    if (ctx.codeReturnMissing && ctx.dynaMem && !ctx.returnMissingNativeNotRTE) {
      runtimeClass=unitContext.getRefUnit(ctx.rteDynamicRuntime, true);
    }
    return flowCode;
	}
	
	protected int innerResolve(int flowCode, Unit unitContext, Mthd mthdContext, Context ctx) { //should never be called as resolve is overwritten
	  return FA_ERROR;
	}
	
	protected void genOutput(Context ctx) {
	  if (!ctx.codeReturnMissing) return;
	  if (ctx.returnMissingNativeNotRTE) ctx.arch.genNativeReturnMissingException();
	  else if (ctx.dynaMem) {
	    ctx.arch.genSaveUnitContext();
	    ctx.arch.genLoadUnitContext(ctx.arch.regClss, -ctx.arch.relocBytes*runtimeClass.index);
	    ctx.arch.genCall(-ctx.arch.relocBytes*ctx.rteRetMissMd.jumpTableIndex, 0, true, ctx.rteRetMissMd.parSize);
	    ctx.arch.genRestUnitContext();
	  }
	  else ctx.arch.genCallConst(ctx.rteRetMissMd, ctx.rteRetMissMd.parSize);
	}
}
