/* Copyright (C) 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package memory;

import osio.BinWriter;

/**
 * ImageContainer: abstract container of an image that should be written to disk
 * 
 * @author S. Frenz
 * @version 090207 added copyright notice
 *  version 080706 added alignBlock
 *  version 071215 removed getByte as it is no longer needed
 *  version 060902 initial version
 */

public class ImageContainer extends MemoryImage {
  public byte[] memBlock;
  public int memBlockLen;
  public int baseAddress, startUnit, startCode;
  
  public void alignBlock(int alignmentMask) {
    memBlockLen=(memBlockLen+alignmentMask)&~alignmentMask;
  }
  
  public void putByte(int addr, byte val) {
    memBlock[addr-baseAddress]=val;
  }
  
  public int getBaseAddress() {
    return baseAddress;
  }
  
  public boolean appendImage(BinWriter w) {
    return w.write(memBlock, 0, memBlockLen);
  }
  
  public void appendImagePart(BinWriter fw, int start, int size) {
    if (size==0) return;
    fw.write(memBlock, start-baseAddress, size);
  }
  
  public int getCRC(int poly, int blockSize) {
    int i, j, stop, crc=-1;
    
    i=memBlockLen%blockSize;
    if (i==0) stop=memBlockLen;
    else stop=memBlockLen+blockSize-i;
    for (i=0; i<stop; i++) {
      if (i<memBlockLen) crc^=((int)memBlock[i])&0xFF;
      for (j=0; j<8; j++) {
        if ((crc&1)!=0) crc=(crc>>>1)^poly;
        else crc=crc>>>1;
      }
    }
    return crc;
  }
}
